from dolfin import *

L = 5
H = 1
mesh = Rectangle(0, 0, L, H, 50, 50)
V = VectorFunctionSpace(mesh, "CG", 2)
Q = FunctionSpace(mesh, "CG", 1)
Z = MixedFunctionSpace([V, Q])


class PeriodicBoundary(SubDomain):
  def inside(self, x, on_boundary):
    return abs(x[0]) < DOLFIN_EPS and on_boundary

  def map(self, x, y):
    y[0] = x[0] - L

no_slip_bottom = DirichletBC(Z.sub(0), (0.0, 0.0), "on_boundary && x[1] < DOLFIN_EPS")
couette_top = DirichletBC(Z.sub(0), (1.0, 0.0), "on_boundary && x[1] > %s - DOLFIN_EPS" % H)
periodic_bc = PeriodicBC(Z, PeriodicBoundary())
bcs = [no_slip_bottom, couette_top, periodic_bc]

def store(z):

  u_pvd = File("results/velocity.pvd")
  p_pvd = File("results/pressure.pvd")
  (u, p) = z.split()

  u_pvd << (u)
  p_pvd << (p)

def main(Re):
  z = Function(Z)
  (u, p) = split(z)

  (u_test, p_test) = split(TestFunction(Z))

  L = inner(grad(u)*u, u_test)*dx + Constant(1.0/Re)*inner(grad(u), grad(u_test))*dx + \
      -div(u_test)*p*dx + p_test*div(u)*dx
  J = derivative(L, z)
  solve(L == 0, z, bcs=bcs, J=J)
  store(z)
  return z

if __name__ == "__main__":
  z = main(350)
