#!/usr/bin/env python

from optparse import OptionParser
import re
import sys
import os.path
from math import sin,cos,pi,acos,atan2,sqrt
from numpy import array

def xyz2latlon(xyz):
  r=sqrt(sum(xyz**2))
  lon = 180.0/pi * atan2(xyz[1],xyz[0])
  lat = 90.0 - 180.0/pi * acos(xyz[2]/r)
  return lat, lon

#####################################################################
# Script starts here.
optparser=OptionParser(usage='usage: %prog [options] <filename_in> <filename_out>',
                       add_help_option=True,
                       description="""This takes a Gmsh 2.0 .msh ascii file 
                       of a mesh on the sphere, and converts its x,y,z coordinates into lat,lon""")

(options, argv) = optparser.parse_args()

if len(argv)<1:
    optparser.print_help()
    sys.exit(1)

if argv[0][-4:]!=".msh":
    sys.stderr.write("Mesh filename must end in .msh\n")
    optparser.print_help()
    sys.exit(1)

filein=file(argv[0], 'r')
fileout=file(argv[1], 'w')

# Header section
line=filein.readline()
assert(line.strip()=="$MeshFormat")
fileout.write(line)

line=filein.readline()
assert(line.strip()in["2 0 8", "2.1 0 8", "2.2 0 8"])
fileout.write(line)

line=filein.readline()
assert(line.strip()=="$EndMeshFormat")
fileout.write(line)


# Nodes section
line=filein.readline()
fileout.write(line)
while line.strip() !="$Nodes":
    line=filein.readline()
    fileout.write(line)
line=filein.readline()
nodecount=int(line)
fileout.write(line)


if nodecount==0:
  sys.stderr.write("ERROR: No nodes found in mesh.\n")
  sys.exit(1)

if nodecount<0:
  sys.stderr.write("ERROR: Negative number of nodes found in mesh.\n")
  sys.exit(1)

for i in range(nodecount):
    # Node syntax
    nodeline = filein.readline().split()
    xyz=array([eval(x) for x in nodeline[1:4]])
    lat,lon=xyz2latlon(xyz)
    fileout.write('%d %f %f 0.0\n'%(i+1,lon,lat))

line=filein.readline()
assert(line.strip()=="$EndNodes")
fileout.write(line)

#copy the rest of the file unmodified
for line in filein:
  fileout.write(line)

filein.close()
fileout.close()
