from dolfin import *
import scipy.sparse
import scipy.sparse.linalg

parameters['linear_algebra_backend'] = 'uBLAS'

L = 5
H = 1
mesh = Rectangle(0, 0, L, H, 50, 50)
V = VectorFunctionSpace(mesh, "CG", 2)
Q = FunctionSpace(mesh, "CG", 1)
Z = MixedFunctionSpace([V, Q])

no_slip_bottom = DirichletBC(Z.sub(0), (0.0, 0.0), "on_boundary && x[1] < DOLFIN_EPS")
no_slip_top = DirichletBC(Z.sub(0), (0.0, 0.0), "on_boundary && x[1] > %s - DOLFIN_EPS" % H)
inlet_bc = DirichletBC(Z.sub(0), (1.0, 0.0), "on_boundary && x[0] < DOLFIN_EPS")
bcs = [no_slip_bottom, no_slip_top, inlet_bc]

def store(z):

  u_pvd = File("results/velocity.pvd")
  p_pvd = File("results/pressure.pvd")
  (u, p) = z.split()

  u_pvd << (u)
  p_pvd << (p)

def sparray(mat):
  from scipy.sparse import dok_matrix
  m_range = mat.local_range(0);
  C = dok_matrix((m_range[1] - m_range[0], mat.size(1)))
  for i, row in enumerate(xrange(*m_range)):
    column, values = mat.getrow(row)
    C[i, column] = values

  return C.tocsr()


def print_eigs(L, z):
  J = derivative(L, z, TrialFunction(Z))
  J = sparray(assemble(J))
  print "Eigenvalues: "
  print scipy.sparse.linalg.eigs(J, k=6, return_eigenvectors=False)

def main(Re):
  z = Function(Z)
  (u, p) = split(z)

  (u_test, p_test) = split(TestFunction(Z))

  L = inner(grad(u)*u, u_test)*dx + Constant(1.0/Re)*inner(grad(u), grad(u_test))*dx + \
      -div(u_test)*p*dx + p_test*div(u)*dx
  J = derivative(L, z)
  solve(L == 0, z, bcs=bcs, J=J)
  store(z)

  print_eigs(L, z)
  return z

if __name__ == "__main__":
  z = main(1000)
