#!/usr/bin/env python
from dolfin import *
import sys

parameters["std_out_all_processes"] = False

mesh = Mesh("doughnut.xml.gz")
facet_labels = MeshFunction("uint", mesh, "doughnut_facets.xml.gz")

# define our function spaces: P2-P1 for velocity-pressure, P1 for advection
V = VectorFunctionSpace(mesh, "CG", 2)
P = FunctionSpace(mesh, "CG", 1)
Q = FunctionSpace(mesh, "CG", 1)
Z = MixedFunctionSpace([V, P, Q])

z = Function(Z, sys.argv[1])

temp = z.split(deepcopy=True)[2]
u    = z.split(deepcopy=True)[0]

dummy = Function(FunctionSpace(mesh ,"R", 0))
dummy.vector()[:] = 1.0
volume = assemble(dummy*dx)
ds = Measure("ds")[facet_labels]
n = FacetNormal(mesh)

print "Mean temperature:    ", assemble(temp*dx) / volume
print "RMS Velocity:        ", sqrt(assemble((u[0]*u[0] + u[1]*u[1])*dx)) * sqrt(1.0/volume)
print "Surface heat flux:   ", assemble(-inner(grad(temp), n)*ds(22))
print "CMB heat flux:       ", assemble(-inner(grad(temp), n)*ds(21))
print "Surface Nusselt:     ", assemble(-inner(grad(temp), n)*ds(22)) / assemble(dummy*ds(22)) * 1.36857
print "CMB Nusselt:         ", assemble(-inner(grad(temp), n)*ds(21)) / assemble(dummy*ds(21)) * 0.75272
